<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * Block class
 *
 * 
 */
class pmxBlock {
	/**
	 * array for instance config settings
	 *
	 * @access private
	 */
	private static $__config = array();
	public static $value = array();
	private static $_bid ;


	/**
	 * pmxBlock::__construct()
	 *
	 * Initialisert die Klasse
	 * 
	 *
	 * @param string $section Config item's name
	 * @return nothing /false
	 */
	public function __construct($parameter)
	{
		self::$_bid=$parameter;
	}

	/**
	 * pmxBlock::__destruct()
	 * Zur Zeit ohne Funktion...
	 */
	public function __destruct()
	{
		// TODO: hier könnte man die Einstellungen gesammelt speichern
	}

	/**
	 * pmxBlock::__get()
	 *
	 * Liest einen einzelnen Wert aus der Konfiguration.
	 * Überladung der get() Funktion.
	 *
	 * gugge: http://www.php.net/manual/de/language.oop5.overloading.php
	 *
	 * @param string $name , Name des Wertes
	 * @return mixed $value, der ausgelesene Wert, oder false, wenn der
	 * Wert nicht existiert
	 */
	public function __get($value_name)
	{
		if (!array_key_exists($value_name,self::$value)) return false;

		return self::$value[$value_name];
	}

	/**
	 * pmxBlock::__set()
	 *
	 * speichet einen einzelnen Wert in der Konfiguration.
	 * Überladung der set() Funktion.
	 *
	 * gugge: http://www.php.net/manual/de/language.oop5.overloading.php
	 *
	 * @param string $name , Name des Wertes
	 *
	 */
	public function __set($value_name, $value)
	{
		if (!array_key_exists($value_name,self::$__config)){
			self::$value[$value_name]=$value;
		} else {
			trigger_error("Variable name reserved: ". htmlentities($value_name),E_USER_WARNING  );
		}
		return ;
	}


	public function __unset($value_name)
	{
		unset(self::$value[$value_name]);
	}

	/**
	 *  @pmxBlock::get()
	 *
	 *  @param [in] $value_name
	 *  @return mixed $value
	 *
	 *  @details Details
	 *
	 */
	static function get($value_name)
	{
		if (!array_key_exists($value_name,self::$value)) return false;

		return self::$value[$value_name];
	}

	static function set($value_name, $value)
	{
		if (!array_key_exists($value_name,self::$__config)){
			self::$value[$value_name]=$value;
		} else {
			trigger_error("Variable name reserved: ". htmlentities($value_name),E_USER_WARNING );
		}
		return ;
	}

	static function set_system($value_name, $value)
	{
		/* hiermit können SystemGlobals dennoch überschrieben werden */
		if (array_key_exists($value_name,self::$__config)){
			self::$value[$value_name]=$value;
		} else {
			trigger_error("Variable name not found: ". htmlentities($value_name),E_USER_WARNING );
		}
		return ;
	}

	/* function __invoke($value_name)
    {
        if (!array_key_exists($value_name,self::$value)) return false;
        
        return self::$value[$value_name];
    }    */

	/**
	 *  @pmxBlock::....()
	 *
	 *  @param [in] $value_name
	 *  @return mixed $value
	 *
	 *  @details Details
	 *
	 */
	public static function __callStatic($value_name, $arguments=NULL)
	{
		// Achtung: Der Wert von $name beachtet die Groß-/Kleinschreibung
		//echo "Rufe die statische Methode '$name' "
		//. implode(', ', $arguments). "\n";
		if ($arguments==NULL) {
			if (!array_key_exists($value_name,self::$value)) return false;

			return self::$value[$value_name];
		}
		if (count($arguments)>1) {
			self::set($value_name,$arguments);
			return;
		}
		self::set($value_name,$arguments[0]);
		return;

	}

	/**
	 *  @pmxBlock::....()
	 *
	 *  @param [in] $value_name
	 *  @return mixed $value
	 *
	 *  @details Details
	 *
	 */
	public function __call($value_name, $arguments=NULL)
	{
		// Achtung: Der Wert von $name beachtet die Groß-/Kleinschreibung
		//echo "Rufe die statische Methode '$name' "
		//. implode(', ', $arguments). "\n";
		if ($arguments==NULL) {
			if (!array_key_exists($value_name,self::$value)) return false;

			return self::$value[$value_name];
		}
		if (count($arguments)>1) {
			self::set($value_name,$arguments);
			return;
		}
		self::set($value_name,$arguments[0]);
		return;

	}
	/**
	 * pmxBlock::read()
	 *
	 *
	 * TODO : Lesen der Configuration
	 *
	 */
	private function read()
	{
		return ;
	}

	/**
	 * pmxBlock::write()
	 *
	 *
	 *  TODO: Schreiben der Configuration
	 *
	 */
	private function write($replace = true)
	{
		return ;
	}

	/**
	 * pmxBlock::get_defaults()
	 *
	 * TODO: Einstellen der Standardwerte
	 *
	 *
	 * @return array , die Werte der gesamten Sektion
	 */
	private function get_defaults()
	{

		return array();
	}

}

/**
 * Base
 * Alias von pmxBlock
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2011
 * @version $Id: Block.php 499 2022-05-11 12:15:10Z PragmaMx $
 * @access public
 */
class Block extends pmxBlock {
	/**
	 * Config::__construct()
	 */
	public function __construct()
	{
		$args = func_get_args();
		parent::__construct($args);
	}
}

?>
