<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Author: Olaf Herfurth / TerraProject  http://www.tecmu.de
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

/*
 * 
 *
 * Usage:
 *          $array = XmlToArray::createArray($xml);
 */

if(!class_exists('XmlToArray')) {
    class XmlToArray {
        private static $xml = null;
        private static $encoding = 'UTF-8';
		
        /**
         * Initialize the root XML node [optional]
			*
         * @param string $version
         * @param string $encoding
         * @param bool $format_output
         */
        public static function init($version = '1.0', $encoding = 'UTF-8', $format_output = true) {
            self::$xml = new DOMDocument($version, $encoding);
            self::$xml->formatOutput = $format_output;
            self::$encoding = $encoding;
        } // END public static function init($version = '1.0', $encoding = 'UTF-8', $format_output = true)
		
        /**
         * Convert an XML to Array
			*
         * @param string $input_xml
         * @throws Exception
         * @return Ambigous DOMDocument <Ambigous, string, multitype:multitype: string multitype:string  Ambigous >
         */
        public static function &createArray($input_xml) {
			//$input_xml= $xml->loadXML($input_xml);
            $input_xml = self::strip_comment($input_xml);
            $xml = self::getXMLRoot();
			
            if(is_string($input_xml)) {
                $parsed = $xml->loadXML($input_xml);
				
                if(!$parsed) {
                    throw new Exception('[XmlToArray] Error parsing the XML string.');
                } // END if(!$parsed)
            } else {
                if(get_class($input_xml) != 'DOMDocument') {
                    throw new Exception('[XmlToArray] The input XML object should be of type: DOMDocument.');
                } // END if(get_class($input_xml) != 'DOMDocument')
				
                $xml = self::$xml = $input_xml;
            } // END if(is_string($input_xml))
			
            $array[$xml->documentElement->tagName] = self::convert($xml->documentElement);
			
            self::$xml = null; // clear the xml node in the class for 2nd time use.
			
            return $array;
        } // END public static function &createArray($input_xml)
		
        /**
         * Convert an Array to XML
			*
         * @param string $node XML as a string or as an object of DOMDocument
         * @return Ambigous <string, multitype:multitype: string multitype:string  unknown >
         */
        private static function &convert($node) {
            $output = array();
			
            switch($node->nodeType) {
                case XML_CDATA_SECTION_NODE:
				$output['@cdata'] = trim($node->textContent);
				break;
				
                case XML_TEXT_NODE:
				$output = trim($node->textContent);
				break;
				
                case XML_ELEMENT_NODE:
				// for each child node, call the covert function recursively
				for($i = 0, $m = $node->childNodes->length; $i < $m; $i++) {
					$child = $node->childNodes->item($i);
					$v = self::convert($child);
					
					if(isset($child->tagName)) {
						$t = $child->tagName;
						
						// assume more nodes of same kind are coming
						if(!isset($output[$t])) {
							$output[$t] = array();
						} // END if(!isset($output[$t]))
						
						$output[$t][] = $v;
					} else {
						//check if it is not an empty text node
						if($v !== '') {
							$output = $v;
						} // END if($v !== '')
					} // END if(isset($child->tagName))
				} // END for($i = 0, $m = $node->childNodes->length; $i < $m; $i++)
				
				if(is_array($output)) {
					// if only one node of its kind, assign it directly instead if array($value);
					foreach($output as $t => $v) {
						if(is_array($v) && count($v) == 1) {
							$output[$t] = $v[0];
						} // END if(is_array($v) && count($v) == 1)
					} // END foreach($output as $t => $v)
					
					if(empty($output)) {
						//for empty nodes
						$output = '';
					} // END if(empty($output))
				} // END if(is_array($output))
				
				// loop through the attributes and collect them
				if($node->attributes->length) {
					$a = array();
					foreach($node->attributes as $attrName => $attrNode) {
						$a[$attrName] = (string) $attrNode->value;
					} // END foreach($node->attributes as $attrName => $attrNode)
					
					// if its an leaf node, store the value in @value instead of directly storing it.
					if(!is_array($output)) {
						$output = array(
						'@value' => $output
						);
					} // END if(!is_array($output))
					
					$output['@attributes'] = $a;
				} // END if($node->attributes->length)
				
				break;
            } // END switch($node->nodeType)
			
            return $output;
        } // END private static function &convert($node)
		
        /**
         * Get the root XML node, if there isn't one, create it.
			*
         * @return NULL
         */
        private static function getXMLRoot() {
            if(empty(self::$xml)) {
                self::init();
            } // END if(empty(self::$xml))
			
            return self::$xml;
        } // END private static function getXMLRoot()
		
        /**
         * Strip the comments out of our XML
			*
         * @param string $input_xml
         * @return string
         */
        private static function strip_comment($input_xml) {
            return preg_replace('/<!--(?:.(?<!--))*-->/', '', $input_xml);
        } // END private static function strip_comment($input_xml)
    } // END class XmlToArray
} // END if(!class_exists('XmlToArray'))
/*
*
* Usage:
*          $xml = ArrayToXml::createXML('root_node_name', $php_array);
*          echo $xml->saveXML();
*/
if(!class_exists('ArrayToXml')) {
    class ArrayToXml {
        private static $xml = null;
        private static $encoding = 'UTF-8';
		
        /**
         * Initialize the root XML node [optional]
			*
         * @param string $version
         * @param string $encoding
         * @param bool $format_output
         */
        public static function init($version = '1.0', $encoding = 'UTF-8', $format_output = true) {
            self::$xml = new DomDocument($version, $encoding);
            self::$xml->formatOutput = $format_output;
            self::$encoding = $encoding;
        } // END public static function init($version = '1.0', $encoding = 'UTF-8', $format_output = true)
		
        /**
         * Convert an Array to XML
			*
         * @param string $node_name name of the root node to be converted
         * @param array $arr aray to be converterd
         * @return DomDocument
         */
        public static function &createXML($node_name, $arr = array()) {
            $xml = self::getXMLRoot();
            $xml->appendChild(self::convert($node_name, $arr));
			
            self::$xml = null; // clear the xml node in the class for 2nd time use.
			
            return $xml;
        } // END public static function &createXML($node_name, $arr = array())
		
        /**
         * Convert an Array to XML
			*
         * @param string $node_name name of the root node to be converted
         * @param array $arr aray to be converterd
         * @throws Exception
         * @return DOMNode
         */
        private static function &convert($node_name, $arr = array()) {
            //print_arr($node_name);
            $xml = self::getXMLRoot();
            $node = $xml->createElement($node_name);
			
            if(is_array($arr)) {
                // get the attributes first.;
                if(isset($arr['@attributes'])) {
                    foreach($arr['@attributes'] as $key => $value) {
                        if(!self::isValidTagName($key)) {
                            throw new Exception('[ArrayToXml] Illegal character in attribute name. attribute: ' . $key . ' in node: ' . $node_name);
                        } // END if(!self::isValidTagName($key))
						
                        $node->setAttribute($key, self::bool2str($value));
                    } // END foreach($arr['@attributes'] as $key => $value)
					
                    unset($arr['@attributes']); //remove the key from the array once done.
                } // END if(isset($arr['@attributes']))
				
                // check if it has a value stored in @value, if yes store the value and return
                // else check if its directly stored as string
                if(isset($arr['@value'])) {
                    $node->appendChild($xml->createTextNode(self::bool2str($arr['@value'])));
					
                    unset($arr['@value']); //remove the key from the array once done.
					
                    //return from recursion, as a note with value cannot have child nodes.
                    return $node;
                } else if(isset($arr['@cdata'])) {
                    $node->appendChild($xml->createCDATASection(self::bool2str($arr['@cdata'])));
					
                    unset($arr['@cdata']); //remove the key from the array once done.
					
                    //return from recursion, as a note with cdata cannot have child nodes.
                    return $node;
                } // END if(isset($arr['@value']))
            } // END if(is_array($arr))
			
            //create subnodes using recursion
            if(is_array($arr)) {
                // recurse to get the node for that key
                foreach($arr as $key => $value) {
                    if(!self::isValidTagName($key)) {
                        throw new Exception('[ArrayToXml] Illegal character in tag name. tag: ' . $key . ' in node: ' . $node_name);
                    } // END if(!self::isValidTagName($key))
					
                    if(is_array($value) && is_numeric(key($value))) {
                        // MORE THAN ONE NODE OF ITS KIND;
                        // if the new array is numeric index, means it is array of nodes of the same kind
                        // it should follow the parent key name
                        foreach($value as $k => $v) {
                            $node->appendChild(self::convert($key, $v));
                        } // END foreach($value as $k => $v)
                    } else {
                        // ONLY ONE NODE OF ITS KIND
                        $node->appendChild(self::convert($key, $value));
                    } // END if(is_array($value) && is_numeric(key($value)))
					
                    unset($arr[$key]); //remove the key from the array once done.
                } // END foreach($arr as $key => $value)
            } // END if(is_array($arr))
			
            // after we are done with all the keys in the array (if it is one)
            // we check if it has any text value, if yes, append it.
            if(!is_array($arr)) {
                $node->appendChild($xml->createTextNode(self::bool2str($arr)));
            } // END if(!is_array($arr))
			
            return $node;
        } // END private static function &convert($node_name, $arr = array())
		
        /**
         * Get the root XML node, if there isn't one, create it.
			*
         * @return string
         */
        private static function getXMLRoot() {
            if(empty(self::$xml)) {
                self::init();
            } // END if(empty(self::$xml))
			
            return self::$xml;
        } // END private static function getXMLRoot()
		
        /**
         * Get string representation of boolean value
			*
         * @param bool $v
         * @return Ambigous <string, unknown>
         */
        private static function bool2str($v) {
            //convert boolean to text value.
            $v = ($v === true) ? 'true' : $v;
            $v = ($v === false) ? 'false' : $v;
			
            return $v;
        } // END private static function bool2str($v)
		
        /**
         * Check if the tag name or attribute name contains illegal characters Ref:
         * http://www.w3.org/TR/xml/#sec-common-syn
			*
         * @param string $tag
         * @return boolean
         */
        private static function isValidTagName($tag) {
            $pattern = '/^[a-z_]+[a-z0-9\:\-\.\_]*[^:]*$/i';
			
            return preg_match($pattern, $tag, $matches) && $matches[0] == $tag;
        } // END private static function isValidTagName($tag)
    } // END class ArrayToXml
} // END if(!class_exists('ArrayToXml'))


?>