<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 *
 *
 * fixes by Marco Andreas (MarcoA/eMmA) http://www.kirmestreffen.de
 */

defined('mxMainFileLoaded') or die('access denied');
pmxBase::set("plugins",0);
$module_name = basename(__DIR__);
mxGetLangfile($module_name);

require_once(PMX_MODULES_DIR . DS . $module_name . DS . 'includes' . DS . 'functions.php');

/**
 * pmxSurveys
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2014
 * @version $Id: index.php 499 2022-05-11 12:15:10Z PragmaMx $
 * @access public
 */
class pmxSurveys {
    private $_tmpl = null;
    private $_polladmin = false;

    /**
     * pmxSurveys::__construct()
     */
    public function __construct()
    {
        global $module_name;

        $GLOBALS['pagetitle'] = _SURVEYS;

        $this->_polladmin = (MX_IS_ADMIN && mxGetAdminPref('radminsurvey'));

        $op = (isset($_REQUEST['op'])) ? $_REQUEST['op'] : '';

        /* Template initialisieren */
        $this->_tmpl = load_class('Template');
        $this->_tmpl->init_path(__FILE__);
        $this->_tmpl->assign('module_name', $module_name);
        $this->_tmpl->assign('_polladmin', $this->_polladmin);

        /* Entscheidung was zu tun ist ;-)) */
        switch (true) {
            case $op == 'poll':
            case !$op && isset($_REQUEST['pollID']); // für veraltete Links :-(
                return $this->poll();

            case $op == 'vote':
                return $this->vote();

            case $op == 'results':
            case $op == _CANCEL: // Kommentar schreiben abbrechen
                return $this->results();

            case $op == 'Reply':
            case $op == _PREVIEW:
            case $op == _OK:
                include_once(__DIR__ . DS . 'includes' . DS . 'comments.php');
                return true;

            default:
                return $this->listpolls();
        }
    }

    /**
     * pmxSurveys::listpolls()
     *
     * @return
     */
    function listpolls()
    {
        global $prefix, $module_name;

        $resultset = sql_query("SELECT p.*, s.title AS arttitle
          FROM `{$prefix}_poll_desc` AS p
            LEFT JOIN {$prefix}_stories AS s ON p.artid = s.sid
          WHERE `pollactive` > 0 " . pmx_multilang_query('planguage', 'AND') . "
          ORDER BY timeStamp DESC");

        $items = array(/* Reihenfolge bestimmen */
            _SURVEYS => array(),
            _SURVEYSATTACHED => array(),
            );

        while ($row = sql_fetch_assoc($resultset)) {
            switch (true) {
                case empty($row['artid']):
                    $type = _SURVEYS;
                    $row['artlink'] = '';
                    break;
                case !empty($row['artid']):
                    $type = _SURVEYSATTACHED;
                    $row['artlink'] = 'modules.php?name=News&amp;file=article&amp;sid=' . $row['artid'];
                    break;
                default:
            }

            $items[$type][] = $row;
        }

        /* Daten dem Template zuweisen */
        $this->_tmpl->assign('items', $items);

        include('header.php');
        $this->_tmpl->display('listpolls.html');
        include_once('footer.php');
    }

    /**
     * pmxSurveys::poll()
     *
     * @return
     */
    function poll()
    {
        global $module_name;

        if (isset($_REQUEST['pollID'])) {
            $pollID = intval($_REQUEST['pollID']);
        } else {
            $pollID = 0;
        }

        $poll = new pmx_poll($pollID);

        switch (true) {
            case !$poll->exist():
            case !$poll->pollactive:
                return mxErrorScreen(_POLLNOTEXIST);
            case !$poll->vote_allowed:
            case $poll->pollactive == 2:
                /* Wenn Umfrage nicht aktiv oder erlaubt ist, nur die Ergebnisse anzeigen */
                return $this->results($poll->id);
        }

        $data = $poll->get();

        $poll_latest = poll_latest();
        $caption = ($poll_latest == $poll->id) ? _CURRENTSURVEY : _SURVEY;

        /* Daten dem Template zuweisen */
        $this->_tmpl->assign($data);
        $this->_tmpl->assign('url_result', $poll->resultslink());
        $this->_tmpl->assign('caption', $caption);

        include('header.php');
        $this->_tmpl->display('vote.html');
        include('footer.php');
    }

    /**
     * pmxSurveys::vote()
     *
     * @return
     */
    function vote()
    {
        global $prefix, $module_name;

        switch (true) {
            case !isset($_POST['pollID']):
            case intval($_POST['pollID']) < 1:
                return $this->listpolls();
            case !isset($_POST['voteID']):
            case intval($_POST['voteID']) < 1:
                return mxRedirect("modules.php?name=$module_name&amp;pollID=" . intval($_POST['pollID']));
            default:
                settype($_POST['pollID'], 'int');
                settype($_POST['voteID'], 'int');
        }

        $poll = new pmx_poll($_POST['pollID']);
        if (!$poll->exist()) {
            return mxErrorScreen(_POLLNOTEXIST);
        }

        $forwarder = $poll->resultslink();

        /* Template für Nachricht initialisieren */
        $template = load_class('Template');
        $this->_tmpl->init_path(__FILE__);
        /* Daten dem Template zuweisen */
        $this->_tmpl->assign('forwarder', $forwarder);
        /* Template auslesen */
        $msgAdd = $this->_tmpl->fetch('votemessage.html');

        /* alle veralteten Datensätze von Nicht-Nur-User Umfragen löschen */
        $past = poll_get_past();
        sql_query("DELETE FROM `{$prefix}_poll_check` WHERE `time` < $past AND `uid`<>0");

        /* hier unterscheiden, ob IP oder UID gesetzt werden muss */
        switch (true) {
            case !$poll->vote_allowed:

                return mxRedirect($forwarder, constant($poll->usermessage) . $msgAdd, 5);
            case $poll->onlyonevote && MX_IS_USER:
                $set[] = " `uid` = " . intval(mxSessionGetVar('user_uid'));
                $set[] = " `ip` = ''";
                break;
            default:
                $set[] = " `uid` = 0";
                $set[] = " `ip` = '" . MX_REMOTE_ADDR . "'";
                break;
        }
        $set[] = " `time` = " . intval(time());
        $set[] = " `pollID` = " . intval($_POST['pollID']);
        $set[] = " `voteID` = " . intval($_POST['voteID']);

        /* tatsächliche Stimmen (+1) für diese Umfrage ermitteln */
        $result = sql_query("SELECT SUM(`optionCount`)+1 FROM `{$prefix}_poll_data` WHERE (`pollID` = " . intval($_POST['pollID']) . ")");
        list($voters) = sql_fetch_row($result);

        switch (true) {
            case !sql_query("INSERT INTO `{$prefix}_poll_check` SET " . implode(",", $set)):
            case !sql_query("UPDATE `{$prefix}_poll_data` SET `optionCount` = `optionCount`+1 WHERE (`pollID` = " . intval($_POST['pollID']) . ") AND (`voteID` = " . intval($_POST['voteID']) . ")"):
            case !sql_query("UPDATE `{$prefix}_poll_desc` SET `voters` = {$voters} WHERE `pollID` = " . intval($_POST['pollID']) . ""):
                return mxRedirect($forwarder, _ISNOTCOUNTED . $msgAdd, 5);
            default:
                return mxRedirect($forwarder, _ISCOUNTED . $msgAdd, 5);
        }
    }

    /**
     * pmxSurveys::results()
     *
     * @return
     */
    function results()
    {
        global $prefix, $module_name;

        if (isset($_REQUEST['pollID'])) {
            $pollID = intval($_REQUEST['pollID']);
        } else {
            $pollID = 0;
        }

        $poll = new pmx_poll($pollID);
        if (!$poll->exist()) {
            return mxErrorScreen(_POLLNOTEXIST);
        }

        $data = $poll->get();

        /* Daten dem Template zuweisen */
        $this->_tmpl->assign($data);
        $this->_tmpl->assign('caption', _CURRENTPOLLRESULTS);

        include('header.php');
        $this->_tmpl->display('results.html');

        if ($data['pollcomments']) {
            include_once(__DIR__ . DS . 'includes' . DS . 'comments.php');
            DisplayTopic($data);
        }
        include('footer.php');
    }
}

$tmp = new pmxSurveys();
$tmp = null;

?>