<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 584 $
 * $Author: PragmaMx $
 * $Date: 2022-08-29 21:51:21 +0200 (Mo, 29. Aug 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * pmx_rss_class
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2010
 * @version $Id: feed.class.php 584 2022-08-29 19:51:21Z PragmaMx $
 * @access public
 */
class pmx_rss_class {
    protected $_cfg_class = null;
    protected $_config = array();
    protected $_defaults = array();
    protected $_module_name = '';

    protected $_formats = array(/* mögliche Formate */
        'RSS' => 'application/rss+xml', // 'RSS0.91'
        'RSS0.91' => 'application/rss+xml',
        'RSS1.0' => 'application/rdf+xml',
        'RSS2.0' => 'application/rss+xml',
        // 'OPML' => 'application/rss+xml',
        // 'PIE0.1' => 'application/rss+xml',
        'ATOM' => 'application/atom+xml',
        'ATOM0.3' => 'application/atom+xml',
        'ATOM1.0' => 'application/atom+xml',
        'HTML' => 'text/html',
        'MBOX' => 'text/plain',
        'JAVASCRIPT' => 'text/javascript',
        );

    const all_mods = 'all';

    /**
     * pmx_rss_class::__construct()
     */
    public function __construct()
    {
        $this->_module_name = basename(dirname(__DIR__));

        mxGetLangfile($this->_module_name);

        $this->_cfg_class = load_class('Config', 'pmx.rss');
        $this->_defaults = $this->_cfg_class->get_defaults();
        $this->_config = $this->_cfg_class->get();

        $this->_config['activated'] = mxModuleActive($this->_module_name);

        $this->_config = array_merge($this->_defaults, $this->_config);
    }

    /**
     * pmx_rss_class::_getallowedmods()
     * aktivierte und für anonym erlaubte Module abfragen
     *
     * @param array $modules
     * @return array
     */
    protected function _getallowedmods($modules)
    {
        global $prefix;
        $modcheck = array();
        $where = '';
        if ($modules) {
            $where = " AND title IN('" . implode("','", (array)$modules) . "')";
        }
        $allmods = sql_query("SELECT title, active
          FROM {$prefix}_modules
          WHERE view=0
            AND active=1" . $where);
        while (list($modulename) = sql_fetch_row($allmods)) {
            $modcheck[] = $modulename;
        }
        return $modcheck;
    }

    /**
     * pmx_rss_class::_modselect()
     *
     * @param mixed $fieldname
     * @param mixed $modlist
     * @param mixed $type
     * @return
     */
    protected function _modselect($fieldname, $modlist, $type)
    {
        $cacheid = __METHOD__ . $type;
        $cache = load_class('Cache');
        // Die Dateierweiterung für die Cache-Dateien (z.B. ".cache")
        $cache->sFileExtension = ".{$this->_module_name}.php";

        if (($hookitems = $cache->read($cacheid)) === false) {
            $hook = load_class('Hook', 'backend');
            if ($type == 'active' && $this->active_sections) {
                /* Checkboxen einschränken auf aktive Module */
                $hook->modlist = $this->active_sections;
            }
            $hook->type = 'modules';
            $hook->only_allowed = false;
            $hookitems = $hook->get(array());
            if (!$hookitems || !is_array($hookitems)) {
                $hookitems = array();
            }
            $cache->write($hookitems, $cacheid);
        }

        /* bei weniger als 2 Modulen braucht man die Auswahl nicht anzeigen */
        if ($type == 'active' && count($hookitems) == 1) {
            return false;
        }

        switch ($type) {
            case 'def':
            case 'all':
                break;
            case 'active':
            default:
                $type = 'all';
        }

        $template = load_class('Template');
        $template->init_path(__DIR__);
        $template->assign('allmodules', $hookitems);
        $template->assign('modlist', $modlist); // die angeklickten...
        $template->assign('fieldname', $fieldname);
        $template->assign('type', $type);

        $modselect = $template->fetch('modselect.html');

        return $modselect;
    }

    /**
     * pmx_rss_class::__get()
     *
     * @param mixed $name
     * @return
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->_config)) {
            return $this->_config[$name];
        }
        $trace = debug_backtrace();
        trigger_error('undefined property \'' . $name . '\' in ' . mx_strip_sysdirs($trace[0]['file']) . ' line ' . $trace[0]['line'], E_USER_NOTICE);
        return null;
    }

    /**
     * pmx_rss_class::__set()
     *
     * @param mixed $name
     * @param mixed $value
     * @return
     */
    public function __set($name, $value)
    {
        $this->_config[$name] = $value;
    }

    /**
     * pmx_rss_class::identical_values()
     * vergleicht 2 eindimensionale Arrays auf Gleichheit
     *
     * @param mixed $arrayA
     * @param mixed $arrayB
     * @return bol
     */
    function identical_values($arrayA , $arrayB)
    {
        sort($arrayA);
        sort($arrayB);

        return $arrayA == $arrayB;
    }
}

?>