<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 584 $
 * $Author: PragmaMx $
 * $Date: 2022-08-29 21:51:21 +0200 (Mo, 29. Aug 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

include_once(__DIR__ . '/feed.class.php');

/**
 * pmxBackend
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2010
 * @version $Id: feed.php 584 2022-08-29 19:51:21Z PragmaMx $
 * @access public
 */
class pmxBackend extends pmx_rss_class {
    /**
     * pmxBackend::__construct()
     */
    public function __construct()
    {
        /*
         * eventuelle Ausgaben zwischenspeichern, um sie am Ende, evtl.
         * wieder als html-Kommentar anzufuegen
         */
        ob_start();

        parent::__construct();

        /* Fehlerausgaben unterdruecken */
        if (pmxDebug::is_mode('enhanced')) {
            pmxDebug::level(pmxDebug::level_logfile);
        }

        /* Einstellungen in die Form der Übergabeparameter konvertieren */
        /* das gibt hübschere Parameter für den generierten Link */
        $defaults = array(
            /* Standardwerte aus Konfiguration lesen */
            'fmt' => $this->default_format,
            'feed' => $this->default_sections,
            'cnt' => $this->default_limit,
            'dln' => $this->default_desclen,
            );

        switch (true) {
            case empty($_GET['feed']):
                $modlist = $defaults['feed'];
                break;
            case $_GET['feed'] == 'all':
                $modlist = $this->active_sections;
                break;
            default:
                $section = preg_replace('#[^a-zA-Z0-9,.\-_]#', '', $_GET['feed']);
                $modlist = preg_split('#\s*[.,]\s*#', $section);
        }

        $settings = array();

        /* nur Daten übernehmen, die auch als default vorhanden sind */
        $settings = array_merge($defaults, $_GET);
        $settings = array_intersect_key($settings, $defaults);

        if ($settings['fmt'] != $defaults['fmt']) {
            if (!array_key_exists($settings['fmt'], $this->_formats)) {
                $settings['fmt'] = $defaults['fmt'];
            }
        }

        if ($settings['cnt'] != $defaults['cnt']) {
            $limit = intval($settings['cnt']);
            if ($limit <= 0 || $limit > $this->max_limit) {
                $settings['cnt'] = $this->max_limit;
            }
        }
        if ($settings['dln'] != $defaults['dln']) {
            $limit = intval($settings['dln']);
            if ($limit < 0 || $limit > $this->max_desclen) {
                $settings['dln'] = $this->max_desclen;
            }
        }

        $settings['content_type'] = $this->_formats[$settings['fmt']];

        $cachetime = intval($this->cachetime);
        if ($cachetime) {
            $cacheid = serialize($this->_config + $settings) . __METHOD__;
            $cache = load_class('Cache');
            // Die Dateierweiterung für die Cache-Dateien (z.B. ".cache")
            $cache->sFileExtension = ".{$this->_module_name}.php";
        }

        if ($cachetime && (($output = $cache->read($cacheid)) !== false)) {
            $output .= PHP_EOL . '<!-- cached -->';
        } else {
            $output = $this->create($settings, $modlist);
            if ($cachetime) {
                $cache->write($output, $cacheid, ($cachetime * 3600)); // x Stunden Cachezeit
            }
        }

        /* Ausgabe, Ende....*/
        header('Content-Type: ' . $settings['content_type'] . '; charset=UTF-8');

        echo $output;
    }

    /**
     * pmxBackend::create()
     *
     * @return
     */
    private function create($settings, $modlist)
    {
        /* leeren Feed erstellen */
        include_once(__DIR__ . '/feedcreator.class.php');
        $feed = new UniversalFeedCreator();

        if ($this->activated) {
            $items = $this->getitems($modlist, $settings['cnt']);
            if ($items) {
                foreach ($items as $key => $value) {
                    $item = new FeedItem();
                    $item->title = strip_tags($value['title']);
                    $item->link = PMX_HOME_URL . '/' . ltrim($value['link'], '/ ');
                    $item->description = $this->preparetext($value['description'], $settings['dln'], $feed);
                    $item->date = intval($value['date']);
                    // $item->source = (strpos($value['source'], PMX_HOME_URL) === 0) ? $value['source'] : PMX_HOME_URL . '/' . ltrim($value['source'], '/ ');
                    $feed->addItem($item);
                }
            }
        }

        /* wenn keine Einträge vorhanden > Fehlermeldung */
        if (!$feed->items) {
            $getname = implode(', ', $modlist);
            $item = $this->moduleerror($getname);
            $feed->addItem($item);
        }

        $feed->descriptionHtmlSyndicated = false;
        $feed->title = htmlspecialchars($this->title);
        $feed->description = htmlspecialchars($this->description);
        $feed->link = PMX_HOME_URL . '/';
        $feed->syndicationURL = PMX_HOME_URL . '/modules.php?name=' . $this->_module_name;
        $feed->contentType = $settings['content_type'];
        // $feed->encoding = "utf-8";
        /* URL zur Logo-Bilddatei (ohne führenden Slash und ohne Domainangabe */
        if (is_file($this->logo)) {
            $feed->image = new FeedImage();
            $feed->image->descriptionHtmlSyndicated = false;
            $feed->image->title = $feed->title;
            $feed->image->url = PMX_HOME_URL . '/' . $this->logo;
            $feed->image->link = PMX_HOME_URL . '/';
            $feed->image->description = sprintf(_RSS_DEFAULTIMGDESC, PMX_HOME_URL);
        }

        $out = $feed->createFeed($settings['fmt']);

        /* Validation, geschuetzte Leerzeichen entfernen */
        $out = str_replace(array('&amp;nbsp;', '&nbsp;'), ' ', $out);

        /* Ersetzen der Links in ein suchmaschinenfreundliches Format. */
        load_class('Modrewrite', false);
        $out = pmxModrewrite::prepare($out, PMX_HOME_URL);

        /* eventuelle Ausgaben, als Kommentar an Ende anfuegen*/
        $temp = trim(ob_get_clean());
        if ($temp) {
            $out . "\n<!-- " . $temp . " -->";
        }

        return trim($out);
    }

    /**
     * pmxBackend::getitems()
     *
     * @param mixed $modname
     * @return
     */
    private function getitems($modlist, $limit)
    {
        /* $modlist = Hook auf die angegebenen Module einschränken, wenn leer werden alle Module gefunden */

        /* 1. existierende Module mit rss-Feed daraus ausfiltern */
        $hook = load_class('Hook', 'backend');
        $hook->modlist = $modlist;
        $hook->only_allowed = false; // soll ja für anonyme sein
        $hook->type = 'modules';
        $modlist = $hook->get(array());
        if ($modlist) {
            /* nur die Schlüssel werden zum Vergleich gebraucht */
            $modlist = array_keys($modlist);
        } else {
            return false;
        }

        /* 2. dann die als Feed aktivierten Module daraus ausfiltern */
        if ($this->active_sections) {
            $modlist = array_intersect($modlist, $this->active_sections);
            if (!$modlist) {
                return false;
            }
        }

        /* 3. In der Datenbank aktivierte und für anonym erlaubte Module abfragen */
        $modlist = $this->_getallowedmods($modlist);
        if (!$modlist) {
            return false;
        }

        $hook->modlist = $modlist;
        $hook->type = 'items';
        $hook->limit = $this->sql_limit($limit);
        $hookitems = $hook->get(array());
        if (!$hookitems) {
            return false;
        }

        $default = array(
            /* Titel des Eintrags */
            'title' => '',
            /* Link zum eigentlichen Eintrags */
            'link' => '',
            /* Beschreibung des Eintrags, HTML wird evtl. durch preparetext() angepasst */
            'description' => '',
            /* Zeitstempel der letzten Eintragsänderung als Unixtimestamp, minus 1 Tag */
            'date' => time() - 86400,
            /* Link zum Modul zu welchem der Eintrag gehört */
            'source' => PMX_HOME_URL,
            /* Modulname des Eintrags */
            'module' => '',
            );

        $items = array();
        foreach ($hookitems as $key => $value) {
            /* Source-Parameter füllen */
            if (empty($value['source'])) {
                if (!empty($value['module'])) {
                    $value['source'] = PMX_HOME_URL . '/modules.php?name=' . $value['module'];
                }
            }

            /* mit Standardwerten sicherstellen dass alle Felder vorhanden sind */
            $value = array_merge($default, $value);

            /* eindeutigen Key zur Sortierung nach Datum */
            $key = strval($value['date'] + (($key + 1) * 0.0001)) . '.' . $value['module'];
            $items[$key] = $value;
        }

        /* absteigende Sortierung nach Datum */
        krsort($items);

        /* Immer nur die eingestellte Anzahl anzeigen */
        list($items) = array_chunk($items, $limit*count($modlist));

        return $items;
    }

    /**
     * pmxBackend::moduleerror()
     *
     * @param mixed $section
     * @return
     */
    protected function moduleerror($section)
    {
        if ($this->activated) {
            $message = sprintf(_RSS_FEEDERROR2, $section) . '<br /><a href="' . $item->link . '">' . $item->link . '</a>';
        } else {
            $message = _RSS_NOTACTIVE;
        }
        $item = new FeedItem();
        $item->title = _RSS_FEEDERROR1;
        $item->link = PMX_HOME_URL . '/modules.php?name=' . $this->_module_name;
        $item->description = $message;
        $item->date = '';
        $item->source = PMX_HOME_URL;
        return $item;
    }

    /**
     * pmxBackend::preparetext()
     *
     * @param mixed $text
     * @return
     */
    protected function preparetext($text, $len, $feedobject)
    {
        $text = trim(str_ireplace('&nbsp;', ' ', $text));

        if ($len) {
            $text = preg_replace('#\s+#', ' ', $text);
            $text = strip_tags($text);
            $text = $feedobject->iTrunc($text, $len);
            return $text;
        }
        $text = $this->rel2abs($text);
        return $text;
    }

    /**
     * pmxBackend::sql_limit()
     * wieviel einzelne Punkte in den jeweiligen feeds anzeigen = _GET[cnt]
     *
     * @return
     */
    protected function sql_limit($limit)
    {
        if ($limit >= 1) {
            return " LIMIT " . $limit;
        }
        return '';
    }

    /**
     * pmxBackend::rel2abs()
     *
     * @param mixed $text
     * @return
     */
    protected function rel2abs($text)
    {
        $pattern = '#(<(?:a|img)[^>]*(?:href|src)=")(' . PMX_BASE_PATH . ')?([^http|ftp|https][^"]*)("[^>]*>)#';
        $replace = '$1' . PMX_HOME_URL . '/$3$4';
        return preg_replace($pattern, $replace, $text);
    }
}

$action = new pmxBackend();
$action = null;

?>